@@ -1,5 +1,6 @@
 import warnings
 
+import cv2
 import math
 import numpy as np
 
@@ -202,6 +203,15 @@ class PolygonAnnotation(Annotation):
 
         # Read the data from rasterio
         data = rasterio_src.read(window=window)
+        
+        # Create mask from polygon
+        polygon_points = np.array([(p.x() - min_x, p.y() - min_y) for p in self.points])
+        mask = np.zeros((window.height, window.width), dtype=np.uint8)
+        cv2.fillPoly(mask, [polygon_points.astype(np.int32)], 1)
+    
+        # Apply mask to each channel
+        for i in range(data.shape[0]):
+            data[i] = data[i] * mask
 
         # Ensure the data is in the correct format for QImage
         data = self._prepare_data_for_qimage(data)
@@ -225,10 +235,11 @@ class PolygonAnnotation(Annotation):
         cropped_polygon = QPolygonF(cropped_points)
         cropped_image_graphic = QGraphicsPolygonItem(cropped_polygon)
 
+        # Create dotted line pen with label color
         color = QColor(self.label.color)
-        color.setAlpha(64)
-        brush = QBrush(color)
-        cropped_image_graphic.setBrush(brush)
+        pen = QPen(color, 3, Qt.DotLine)
+        cropped_image_graphic.setPen(pen)
+        cropped_image_graphic.setBrush(QBrush(Qt.NoBrush)) 
         cropped_image_graphic.update()
 
         return cropped_image_graphic
