@@ -5,19 +5,23 @@ warnings.filterwarnings("ignore", category=DeprecationWarning)
 import gc
 
 import torch
+from torch.cuda import empty_cache
 from autodistill.detection import CaptionOntology
 
-from qtrangeslider import QRangeSlider
+from superqt import QRangeSlider
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QApplication, QComboBox, QDialog,
                              QFormLayout, QHBoxLayout, QLabel, QLineEdit,
                              QMessageBox, QPushButton, QSlider, QVBoxLayout, QGroupBox)
 
-from torch.cuda import empty_cache
 
 from coralnet_toolbox.QtProgressBar import ProgressBar
+
 from coralnet_toolbox.ResultsProcessor import ResultsProcessor
-from coralnet_toolbox.utilities import rasterio_to_numpy
+
+from coralnet_toolbox.utilities import open_image
+
+from coralnet_toolbox.Icons import get_icon
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -43,7 +47,8 @@ class DeployModelDialog(QDialog):
         self.main_window = main_window
         self.label_window = main_window.label_window
         self.annotation_window = main_window.annotation_window
-        
+
+        self.setWindowIcon(get_icon("coral.png"))
         self.setWindowTitle("AutoDistill Deploy Model")
         self.resize(400, 325)
 
@@ -61,7 +66,7 @@ class DeployModelDialog(QDialog):
 
         # Create the layout
         self.layout = QVBoxLayout(self)
-        
+
         # Setup the info layout
         self.setup_info_layout()
         # Setup the model layout
@@ -74,7 +79,7 @@ class DeployModelDialog(QDialog):
         self.setup_buttons_layout()
         # Setup the status layout
         self.setup_status_layout()
-        
+
     def showEvent(self, event):
         """
         Handle the show event to update label options and sync uncertainty threshold.
@@ -94,14 +99,14 @@ class DeployModelDialog(QDialog):
         """
         group_box = QGroupBox("Information")
         layout = QVBoxLayout()
-        
+
         # Create a QLabel with explanatory text and hyperlink
         info_label = QLabel("Choose a model to deploy and use.")
-        
+
         info_label.setOpenExternalLinks(True)
         info_label.setWordWrap(True)
         layout.addWidget(info_label)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
@@ -111,11 +116,11 @@ class DeployModelDialog(QDialog):
         """
         group_box = QGroupBox("Model Selection")
         layout = QVBoxLayout()
-        
+
         self.model_dropdown = QComboBox()
         self.model_dropdown.addItems(["GroundingDINO-SwinT", "GroundingDINO-SwinB"])
         layout.addWidget(self.model_dropdown)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
@@ -125,20 +130,20 @@ class DeployModelDialog(QDialog):
         """
         group_box = QGroupBox("Ontology Mapping")
         layout = QVBoxLayout()
-        
+
         add_remove_layout = QHBoxLayout()
-        
+
+        self.add_button = QPushButton("Add")
+        self.add_button.clicked.connect(self.add_ontology_pair)
+        add_remove_layout.addWidget(self.add_button)
+
         self.remove_button = QPushButton("Remove")
         self.remove_button.clicked.connect(self.remove_ontology_pair)
         add_remove_layout.addWidget(self.remove_button)
-        
-        self.add_button = QPushButton("Add")  
-        self.add_button.clicked.connect(self.add_ontology_pair)
-        add_remove_layout.addWidget(self.add_button)
-        
+
         layout.addLayout(add_remove_layout)
         self.ontology_layout = layout
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
@@ -160,7 +165,7 @@ class DeployModelDialog(QDialog):
         self.uncertainty_threshold_label = QLabel(f"{self.uncertainty_thresh:.2f}")
         layout.addRow("Uncertainty Threshold", self.uncertainty_threshold_slider)
         layout.addRow("", self.uncertainty_threshold_label)
-        
+
         # IoU threshold controls
         self.iou_thresh = self.main_window.get_iou_thresh()
         self.iou_threshold_slider = QSlider(Qt.Horizontal)
@@ -172,7 +177,7 @@ class DeployModelDialog(QDialog):
         self.iou_threshold_label = QLabel(f"{self.iou_thresh:.2f}")
         layout.addRow("IoU Threshold", self.iou_threshold_slider)
         layout.addRow("", self.iou_threshold_label)
-        
+
         # Area threshold controls
         min_val, max_val = self.main_window.get_area_thresh()
         self.area_thresh_min = int(min_val * 100)
@@ -186,47 +191,47 @@ class DeployModelDialog(QDialog):
         self.area_threshold_label = QLabel(f"{self.area_thresh_min:.2f} - {self.area_thresh_max:.2f}")
         layout.addRow("Area Threshold", self.area_threshold_slider)
         layout.addRow("", self.area_threshold_label)
-        
+
         # SAM dropdown
         self.use_sam_dropdown = QComboBox()
         self.use_sam_dropdown.addItems(["False", "True"])
         self.use_sam_dropdown.currentIndexChanged.connect(self.is_sam_model_deployed)
         layout.addRow("Use SAM for creating Polygons:", self.use_sam_dropdown)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
     def setup_status_layout(self):
         """
         Setup status display in a group box.
         """
         group_box = QGroupBox("Status")
         layout = QVBoxLayout()
-        
+
         self.status_bar = QLabel("No model loaded")
         layout.addWidget(self.status_bar)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
     def setup_buttons_layout(self):
         """
         Setup action buttons in a group box.
         """
         group_box = QGroupBox("Actions")
         layout = QHBoxLayout()
-        
+
         load_button = QPushButton("Load Model")
         load_button.clicked.connect(self.load_model)
         layout.addWidget(load_button)
-        
+
         deactivate_button = QPushButton("Deactivate Model")
         deactivate_button.clicked.connect(self.deactivate_model)
         layout.addWidget(deactivate_button)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
     def initialize_uncertainty_threshold(self):
         """Initialize the uncertainty threshold slider with the current value"""
         current_value = self.main_window.get_uncertainty_thresh()
@@ -238,7 +243,7 @@ class DeployModelDialog(QDialog):
         current_value = self.main_window.get_iou_thresh()
         self.iou_threshold_slider.setValue(int(current_value * 100))
         self.iou_thresh = current_value
-        
+
     def initialize_area_threshold(self):
         """Initialize the area threshold range slider"""
         current_min, current_max = self.main_window.get_area_thresh()
@@ -256,7 +261,7 @@ class DeployModelDialog(QDialog):
     def update_iou_label(self, value):
         """Update IoU threshold and label"""
         value = value / 100.0
-        self.iou_thresh = value 
+        self.iou_thresh = value
         self.main_window.update_iou_thresh(value)
         self.iou_threshold_label.setText(f"{value:.2f}")
 
@@ -309,14 +314,14 @@ class DeployModelDialog(QDialog):
             # Remove the layout
             item = self.ontology_layout.itemAt(self.ontology_layout.count() - 2)
             item.layout().deleteLater()
-            
+
     def is_sam_model_deployed(self):
         """
         Check if the SAM model is deployed and update the checkbox state accordingly.
         """
         if not hasattr(self.main_window, 'sam_deploy_model_dialog'):
             return False
-        
+
         self.sam_dialog = self.main_window.sam_deploy_model_dialog
 
         if not self.sam_dialog.loaded_model:
@@ -324,21 +329,27 @@ class DeployModelDialog(QDialog):
             QMessageBox.critical(self, "Error", "Please deploy the SAM model first.")
             return False
 
-        return True 
+        return True
 
     def load_model(self):
         """
         Load the selected model with the current configuration.
         """
+        # Get the ontology mapping
+        ontology_mapping = self.get_ontology_mapping()
+
+        if not ontology_mapping:
+            QMessageBox.critical(self,
+                                 "Error",
+                                 "Please provide at least one ontology mapping.")
+            return
+
         # Make cursor busy
         QApplication.setOverrideCursor(Qt.WaitCursor)
         # Show a progress bar
         progress_bar = ProgressBar(self.annotation_window, title="Loading Model")
         progress_bar.show()
         try:
-            # Get the ontology mapping
-            ontology_mapping = self.get_ontology_mapping()
-
             # Set the ontology
             self.ontology = CaptionOntology(ontology_mapping)
             # Set the class mapping
@@ -398,69 +409,112 @@ class DeployModelDialog(QDialog):
                                               text_threshold=0.025,
                                               model=model)
 
+    # TODO Error: 'list' object has no attribute 'xyxy'
     def predict(self, image_paths=None):
         """
         Make Autodistill predictions on the given inputs
         """
-        if not self.loaded_model:
-            return
-   
+        if self.loaded_model is None:
+            return  # Early exit if there is no model loaded
+
+        # Create a result processor
+        results_processor = ResultsProcessor(
+            self.main_window,
+            self.class_mapping,
+            uncertainty_thresh=self.main_window.get_uncertainty_thresh(),
+            iou_thresh=self.main_window.get_iou_thresh(),
+            min_area_thresh=self.main_window.get_area_thresh_min(),
+            max_area_thresh=self.main_window.get_area_thresh_max()
+        )
+
+        # Use current image if image_paths is not provided.
         if not image_paths:
             image_paths = [self.annotation_window.current_image_path]
-            
+
         # Make cursor busy
         QApplication.setOverrideCursor(Qt.WaitCursor)
-            
-        progress_bar = ProgressBar(self.annotation_window, title=f"Making {self.model_name} Predictions")
-        progress_bar.show()
-        progress_bar.start_progress(len(image_paths))
 
-        for image_path in image_paths:
-            # Open the image
-            image = self.main_window.image_window.rasterio_open(image_path)
-            image = rasterio_to_numpy(image)
-            # Predict the image
-            results = self.loaded_model.predict(image)
-            
-            # Create a results processor
-            results_processor = ResultsProcessor(self.main_window, 
-                                                 self.class_mapping,
-                                                 uncertainty_thresh=self.main_window.get_uncertainty_thresh(),
-                                                 iou_thresh=self.main_window.get_iou_thresh(),
-                                                 min_area_thresh=self.main_window.get_area_thresh_min(),
-                                                 max_area_thresh=self.main_window.get_area_thresh_max())
-            
-            results = results_processor.from_supervision(results, image, image_path, self.class_mapping)
-            
-            # Update the progress bar
-            progress_bar.update_progress()
+        try:
+            # Loop through the image paths
+            for image_path in image_paths:
+                inputs = self._get_inputs(image_path)
+                if inputs is None:
+                    continue
+
+                results = self._apply_model(inputs)
+                results = self._update_results(results_processor, results, inputs, image_path)
+                results = self._apply_sam(results, image_path)
+                results = self._apply_tile_postprocessing(results)
+                self._process_results(results_processor, results)
+        except Exception as e:
+            print("An error occurred during prediction:", e)
+        finally:
+            QApplication.restoreOverrideCursor()
 
-            if self.use_sam_dropdown.currentText() == "True":
-                # Apply SAM to the detection results
-                results = self.sam_dialog.predict_from_results(results, self.class_mapping)
-                # Process the segmentation results
-                results_processor.process_segmentation_results(results)
-            else:
-                # Process the detection results
-                results_processor.process_detection_results(results)
-                
-        # Stop the progress bar
-        progress_bar.stop_progress()
-        progress_bar.close()
-                
-        # Make cursor normal
-        QApplication.restoreOverrideCursor()
         gc.collect()
-        empty_cache()
+        empty_cache()  # Assuming this function is defined elsewhere
+
+    def _get_inputs(self, image_path):
+        """Get the inputs for the model prediction."""
+        # Check if tile inference tool is enabled
+        if self.main_window.tile_inference_tool_action.isChecked():
+            self.loaded_model.names = self.class_mapping
+            inputs = self.main_window.tile_processor.make_crops(self.loaded_model, image_path)
+            if not inputs:
+                return None
+        else:
+            inputs = open_image(image_path)
+        return inputs
+
+    def _apply_model(self, inputs):
+        """Apply the model to the inputs."""
+        return self.loaded_model.predict(inputs)
+
+    def _update_results(self, results_processor, results, inputs, image_path):
+        """Update the results to match Ultralytics format."""
+        return results_processor.from_supervision(results,
+                                                  inputs,
+                                                  image_path,
+                                                  self.class_mapping)
+
+    def _apply_sam(self, results, image_path):
+        """Apply SAM to the results if needed."""
+        # Check if SAM model is deployed
+        if self.use_sam_dropdown.currentText() == "True":
+            self.task = 'segment'
+            results = self.sam_dialog.predict_from_results(results, self.class_mapping, image_path)
+        else:
+            self.task = 'detect'
+
+        return results
+
+    def _apply_tile_postprocessing(self, results):
+        """Apply tile postprocessing if needed."""
+        # Check if tile inference tool is enabled
+        if self.main_window.tile_inference_tool_action.isChecked():
+            results = self.main_window.tile_processor.detect_them(results, self.task == 'segment')
+        return results
+
+    def _process_results(self, result_processor, results):
+        """Process the results using the result processor."""
+        # Process the detections
+        if self.task == 'segment':
+            result_processor.process_segmentation_results(results)
+        else:
+            result_processor.process_detection_results(results)
 
     def deactivate_model(self):
         """
         Deactivate the currently loaded model and clean up resources.
         """
+        # Clear the model
         self.loaded_model = None
         self.model_name = None
+        # Clear cache
         gc.collect()
         torch.cuda.empty_cache()
+        # Untoggle all tools
         self.main_window.untoggle_all_tools()
+        # Update status bar
         self.status_bar.setText("No model loaded")
-        QMessageBox.information(self, "Model Deactivated", "Model deactivated")
\ No newline at end of file
+        QMessageBox.information(self, "Model Deactivated", "Model deactivated")
