@@ -12,7 +12,7 @@ from PyQt5.QtGui import QBrush, QColor
 from PyQt5.QtWidgets import (QFileDialog, QApplication, QMessageBox, QCheckBox,
                              QVBoxLayout, QLabel, QLineEdit, QDialog, QHBoxLayout,
                              QPushButton, QFormLayout, QDialogButtonBox, QDoubleSpinBox,
-                             QGroupBox, QTableWidget, QTableWidgetItem)
+                             QGroupBox, QTableWidget, QTableWidgetItem, QButtonGroup, QRadioButton)
 
 from coralnet_toolbox.Annotations.QtRectangleAnnotation import RectangleAnnotation
 from coralnet_toolbox.Annotations.QtPolygonAnnotation import PolygonAnnotation
@@ -46,7 +46,7 @@ class Base(QDialog):
         self.selected_labels = []
         self.selected_annotations = []
         self.updating_summary_statistics = False
-        
+
         self.output_dir = None
         self.dataset_name = None
         self.train_ratio = 0.7
@@ -60,6 +60,7 @@ class Base(QDialog):
         self.setup_output_layout()
         self.setup_ratio_layout()
         self.setup_annotation_layout()
+        self.setup_options_layout()
         self.setup_table_layout()
         self.setup_status_layout()
         self.setup_button_layout()
@@ -76,7 +77,7 @@ class Base(QDialog):
         self.update_annotation_type_checkboxes()
         self.populate_class_filter_list()
         self.update_summary_statistics()
-        
+
     def setup_info_layout(self):
         """
         Set up the layout and widgets for the info layout.
@@ -170,6 +171,37 @@ class Base(QDialog):
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
+    def setup_options_layout(self):
+        """Setup the image options layout."""
+        group_box = QGroupBox("Image Options")
+        layout = QVBoxLayout()
+
+        # Create a button group for the image checkboxes
+        self.image_options_group = QButtonGroup(self)
+
+        self.all_images_radio = QRadioButton("All Images")
+        self.filtered_images_radio = QRadioButton("Filtered Images")
+
+        # Add the radio buttons to the button group
+        self.image_options_group.addButton(self.all_images_radio)
+        self.image_options_group.addButton(self.filtered_images_radio)
+
+        # Ensure only one radio button can be checked at a time
+        self.image_options_group.setExclusive(True)
+
+        # Set the default radio button
+        self.all_images_radio.setChecked(True)
+
+        # Connect radio button signals
+        self.all_images_radio.toggled.connect(self.update_image_selection)
+        self.filtered_images_radio.toggled.connect(self.update_image_selection)
+
+        layout.addWidget(self.all_images_radio)
+        layout.addWidget(self.filtered_images_radio)
+
+        group_box.setLayout(layout)
+        self.layout.addWidget(group_box)
+
     def setup_table_layout(self):
         """Setup the label counts table layout."""
         group_box = QGroupBox("Annotation Table")
@@ -268,7 +300,7 @@ class Base(QDialog):
         """
         # Get the label objects for the selected labels
         class_mapping = {}
-        
+
         for label in self.main_window.label_window.labels:
             if label.short_label_code in self.selected_labels:
                 class_mapping[label.short_label_code] = label.to_dict()
@@ -328,7 +360,11 @@ class Base(QDialog):
 
         # Filter annotations based on the selected labels
         annotations = [a for a in filtered_annotations if a.label.short_label_code in self.selected_labels]
-        
+
+        # Filter annotations based on the selected image option
+        if self.filtered_images_radio.isChecked():
+            annotations = [a for a in annotations if a.image_path in self.image_window.filtered_image_paths]
+
         return annotations
 
     def populate_class_filter_list(self):
@@ -493,6 +529,13 @@ class Base(QDialog):
 
         return True
 
+    def update_image_selection(self):
+        """
+        Update the table based on the selected image option.
+        """
+        self.selected_annotations = self.filter_annotations()
+        self.update_summary_statistics()
+
     def update_summary_statistics(self):
         """
         Update the summary statistics for the dataset creation.
@@ -556,7 +599,7 @@ class Base(QDialog):
         self.ready_label.setText("✅ Ready" if (self.ready_status and self.split_status) else "❌ Not Ready")
 
         self.updating_summary_statistics = False
-        
+
     def is_ready(self):
         """Check if the dataset is ready to be created."""
         # Extract the input values, store them in the class variables
@@ -584,9 +627,9 @@ class Base(QDialog):
                                 "Input Error",
                                 "Train, Validation, and Test ratios must sum to 1.0")
             return False
-        
+
         return True
-    
+
     def accept(self):
         """
         Handle the OK button click event to create the dataset.
@@ -596,7 +639,7 @@ class Base(QDialog):
 
         # Create the output folder
         output_dir_path = os.path.join(self.output_dir, self.dataset_name)
-        
+
         # Check if the output directory exists
         if os.path.exists(output_dir_path):
             reply = QMessageBox.question(self,
@@ -634,9 +677,9 @@ class Base(QDialog):
                                 "Dataset Created",
                                 "Dataset has been successfully created.")
         super().accept()
-        
-    def create_dataset(self):
+
+    def create_dataset(self, output_dir_path):
+        raise NotImplementedError("Method must be implemented in the subclass.")
+
+    def process_annotations(self, annotations, split_dir, split):
         raise NotImplementedError("Method must be implemented in the subclass.")
-    
-    def process_annotations(self):
-        raise NotImplementedError("Method must be implemented in the subclass.")
\ No newline at end of file
