@@ -63,6 +63,7 @@ class AnnotationWindow(QGraphicsView):
         self.cursor_annotation = None
 
         self.annotations_dict = {}  # Dictionary to store annotations by UUID
+        self.image_annotations_dict = {}  # Dictionary to store annotations by image path
 
         self.selected_annotations = []  # Stores the selected annotations
         self.selected_label = None  # Flag to check if an active label is set
@@ -192,7 +193,14 @@ class AnnotationWindow(QGraphicsView):
     def set_annotation_location(self, annotation_id, new_center_xy: QPointF):
         if annotation_id in self.annotations_dict:
             annotation = self.annotations_dict[annotation_id]
+            # Disconnect the confidence window from the annotation, so it won't update while moving
+            annotation.annotationUpdated.disconnect(self.main_window.confidence_window.display_cropped_image)
             annotation.update_location(new_center_xy)
+            # Connect the confidence window back to the annotation
+            annotation.annotationUpdated.connect(self.main_window.confidence_window.display_cropped_image)
+            # Create and display the cropped image in the confidence window
+            annotation.create_cropped_image(self.rasterio_image)
+            self.main_window.confidence_window.display_cropped_image(annotation)
 
     def set_annotation_size(self, size=None, delta=0):
         if size is not None:
@@ -206,6 +214,10 @@ class AnnotationWindow(QGraphicsView):
             annotation = self.selected_annotations[0]
             if not self.is_annotation_moveable(annotation):
                 return
+            
+            # Disconnect the confidence window from the annotation, so it won't update while resizing
+            annotation.annotationUpdated.disconnect(self.main_window.confidence_window.display_cropped_image)
+            
             if isinstance(annotation, PatchAnnotation):
                 annotation.update_annotation_size(self.annotation_size)
                 if self.cursor_annotation:
@@ -221,7 +233,11 @@ class AnnotationWindow(QGraphicsView):
                 if self.cursor_annotation:
                     self.cursor_annotation.update_annotation_size(scale_factor)
 
+            # Create and display the cropped image in the confidence window
             annotation.create_cropped_image(self.rasterio_image)
+            # Connect the confidence window back to the annotation
+            annotation.annotationUpdated.connect(self.main_window.confidence_window.display_cropped_image)
+            # Display the cropped image in the confidence window
             self.main_window.confidence_window.display_cropped_image(annotation)
 
         # Only emit if 1 or no annotations are selected
@@ -274,6 +290,10 @@ class AnnotationWindow(QGraphicsView):
         # Clean up
         self.clear_scene()
 
+        # Check that the image path is valid
+        if image_path not in self.main_window.image_window.images:
+            return
+
         # Set the image representations
         self.image_pixmap = QPixmap(self.main_window.image_window.images[image_path])
         self.rasterio_image = self.main_window.image_window.rasterio_images[image_path]
@@ -317,6 +337,10 @@ class AnnotationWindow(QGraphicsView):
         return 0, 0
 
     def center_on_annotation(self, annotation):
+        # Create graphics item if it doesn't exist
+        if not annotation.graphics_item:
+            annotation.create_graphics_item(self.scene)
+            
         # Get the bounding rect of the annotation in scene coordinates
         annotation_rect = annotation.graphics_item.boundingRect()
         annotation_center = annotation_rect.center()
@@ -367,10 +391,11 @@ class AnnotationWindow(QGraphicsView):
                 # Select the first annotation
                 new_index = 0
 
-            # Select the new annotation
-            self.select_annotation(annotations[new_index])
-            # Center the view on the new annotation
-            self.center_on_annotation(annotations[new_index])
+            if 0 <= new_index < len(annotations):
+                # Select the new annotation
+                self.select_annotation(annotations[new_index])
+                # Center the view on the new annotation
+                self.center_on_annotation(annotations[new_index])
 
     def select_annotation(self, annotation, ctrl_pressed=False):
         if not ctrl_pressed:
@@ -441,48 +466,48 @@ class AnnotationWindow(QGraphicsView):
         # Connect essential update signals
         annotation.selected.connect(self.select_annotation)
         annotation.annotationDeleted.connect(self.delete_annotation)
+        annotation.annotationUpdated.connect(self.main_window.confidence_window.display_cropped_image)
+        # Update the view   
         self.viewport().update()
 
-    def load_annotations(self):
-        # Initialize the progress bar
+    def load_annotations(self, image_path=None, annotations=None):
+        """
+        Loads annotations for the current image by default or for a given image and annotations.
+        """
+        # Crop annotations (if image_path and annotations are provided, they are used)
+        annotations = self.crop_annotations(image_path, annotations)
+        total = len(annotations)
+        
+        # Make cursor busy
+        QApplication.setOverrideCursor(Qt.WaitCursor)
         progress_bar = ProgressBar(self, title="Loading Annotations")
         progress_bar.show()
+        progress_bar.start_progress(total)
+        
+        try:
+            # Load each annotation and update progress
+            for idx, annotation in enumerate(annotations):
+                if progress_bar.wasCanceled():
+                    break
+                
+                # Load the annotation
+                self.load_annotation(annotation)
 
-        # Crop all the annotations for current image (if not already cropped)
-        annotations = self.crop_image_annotations(return_annotations=True)
-        progress_bar.start_progress(len(annotations))
-
-        # Connect update signals for all the annotations
-        for idx, annotation in enumerate(annotations):
-            if progress_bar.wasCanceled():
-                break
-            self.load_annotation(annotation)
-            progress_bar.update_progress()
-
-        progress_bar.stop_progress()
-        progress_bar.close()
-
-        QApplication.processEvents()
-        self.viewport().update()
-
-    def load_these_annotations(self, image_path, annotations):
-        # Initialize the progress bar
-        progress_bar = ProgressBar(self, title="Loading Annotations")
-        progress_bar.show()
-
-        # Crop all the annotations for current image (if not already cropped)
-        annotations = self.crop_these_image_annotations(image_path, annotations)
-        progress_bar.start_progress(len(annotations))
-
-        # Connect update signals for all the annotations
-        for annotation in annotations:
-            if progress_bar.wasCanceled():
-                break
-            self.load_annotation(annotation)
-            progress_bar.update_progress()
-
-        progress_bar.stop_progress()
-        progress_bar.close()
+                # Update every 10% of the annotations (or for each item if total is small)
+                if total > 10:
+                    if idx % (total // 10) == 0:
+                        progress_bar.update_progress_percentage((idx / total) * 100)
+                else:
+                    progress_bar.update_progress_percentage((idx / total) * 100)
+                    
+        except Exception as e:
+            QMessageBox.critical(self, "Error", str(e))
+            
+        finally:
+            # Restore the cursor
+            QApplication.restoreOverrideCursor()
+            progress_bar.stop_progress()
+            progress_bar.close()
 
         QApplication.processEvents()
         self.viewport().update()
@@ -491,12 +516,7 @@ class AnnotationWindow(QGraphicsView):
         if not image_path:
             image_path = self.current_image_path
 
-        annotations = []
-        for annotation_id, annotation in self.annotations_dict.items():
-            if annotation.image_path == image_path:
-                annotations.append(annotation)
-
-        return annotations
+        return self.image_annotations_dict.get(image_path, [])
 
     def get_image_review_annotations(self, image_path=None):
         if not image_path:
@@ -509,69 +529,48 @@ class AnnotationWindow(QGraphicsView):
 
         return annotations
 
-    def crop_image_annotations(self, image_path=None, return_annotations=False):
+    def crop_annotations(self, image_path=None, annotations=None, return_annotations=True, verbose=True):
+        """
+        Crop annotations for the specified image.
+
+        If annotations is None, crop all annotations associated with the image (using self.get_image_annotations).
+        If return_annotations is True, returns the list of annotations after cropping.
+        """
+        # Make cursor busy
+        QApplication.setOverrideCursor(Qt.WaitCursor)
+        
         if not image_path:
-            # Set the image path if not provided
             image_path = self.current_image_path
-        # Get the annotations for the image
-        annotations = self.get_image_annotations(image_path)
-        self._crop_annotations_batch(image_path, annotations)
-        # Return the annotations if flag is set
+
+        if annotations is None:
+            annotations = self.get_image_annotations(image_path)
+
+        progress_bar = None
+        if verbose:
+            progress_bar = ProgressBar(self, title="Cropping Annotations")
+            progress_bar.show()
+            progress_bar.start_progress(len(annotations))
+
+        try:
+            rasterio_image = self.main_window.image_window.rasterio_open(image_path)
+            for annotation in annotations:
+                if not getattr(annotation, "cropped_image", False):
+                    annotation.create_cropped_image(rasterio_image)
+                if verbose:
+                    progress_bar.update_progress()
+                    
+        except Exception as e:
+            QMessageBox.critical(self, "Error", str(e))
+            
+        finally:
+            QApplication.restoreOverrideCursor()
+            if verbose:
+                progress_bar.stop_progress()
+                progress_bar.close()
+
         if return_annotations:
             return annotations
 
-    def crop_these_image_annotations(self, image_path, annotations, linear=False):
-        if linear:
-            self._crop_annotations_batch_linear(image_path, annotations)
-        else:
-            self._crop_annotations_batch(image_path, annotations)
-        return annotations
-
-    def _crop_annotations_batch_linear(self, image_path, annotations):
-        # Create a progress bar
-        progress_bar = ProgressBar(self, title="Cropping Annotations")
-        progress_bar.show()
-        progress_bar.start_progress(len(annotations))
-
-        # Get the rasterio representation
-        rasterio_image = self.main_window.image_window.rasterio_open(image_path)
-        # Loop through the annotations, crop the image if not already cropped
-        for annotation in annotations:
-            if not annotation.cropped_image:
-                annotation.create_cropped_image(rasterio_image)
-            progress_bar.update_progress()
-
-        progress_bar.stop_progress()
-        progress_bar.close()
-
-    def _crop_annotations_batch(self, image_path, annotations):
-        # Create a progress bar
-        progress_bar = ProgressBar(self, title="Cropping Annotations")
-        progress_bar.show()
-        progress_bar.start_progress(len(annotations))
-
-        # Create a lock for thread-safe access to the rasterio dataset
-        lock = threading.Lock()
-
-        # Get the rasterio representation
-        rasterio_image = self.main_window.image_window.rasterio_open(image_path)
-
-        def crop_annotation(annotation):
-            with lock:
-                if not annotation.cropped_image:
-                    annotation.create_cropped_image(rasterio_image)
-            return annotation
-
-        # Use ThreadPoolExecutor to crop images in parallel
-        with ThreadPoolExecutor(max_workers=4) as executor:
-            futures = [executor.submit(crop_annotation, annotation) for annotation in annotations]
-            for future in as_completed(futures):
-                future.result()  # Ensure any exceptions are raised
-                progress_bar.update_progress()
-
-        progress_bar.stop_progress()
-        progress_bar.close()
-
     def add_annotation(self, scene_pos: QPointF = None):
         if not self.selected_label:
             QMessageBox.warning(self,
@@ -595,22 +594,43 @@ class AnnotationWindow(QGraphicsView):
             self.toggle_cursor_annotation()
             return
 
-        annotation.create_graphics_item(self.scene)
-        annotation.create_cropped_image(self.rasterio_image)
-
         # Connect update signals
         annotation.selected.connect(self.select_annotation)
         annotation.annotationDeleted.connect(self.delete_annotation)
         annotation.annotationUpdated.connect(self.main_window.confidence_window.display_cropped_image)
-
-        self.annotations_dict[annotation.id] = annotation
+        
+        # Create the graphics item
+        annotation.create_graphics_item(self.scene)
+        annotation.create_cropped_image(self.rasterio_image)
+        
+        # Display the cropped image in the confidence window
         self.main_window.confidence_window.display_cropped_image(annotation)
+
+        # Add to annotation dict
+        self.add_annotation_to_dict(annotation)
+        
+        # Update the table in ImageWindow
         self.annotationCreated.emit(annotation.id)
 
+    def add_annotation_to_dict(self, annotation):       
+        # Add to annotation dict
+        self.annotations_dict[annotation.id] = annotation
+        # Add to image annotations dict (if not already present)
+        if annotation.image_path not in self.image_annotations_dict:
+            self.image_annotations_dict[annotation.image_path] = []
+        if annotation not in self.image_annotations_dict[annotation.image_path]:
+            self.image_annotations_dict[annotation.image_path].append(annotation)
+
     def delete_annotation(self, annotation_id):
         if annotation_id in self.annotations_dict:
             # Get the annotation from dict
             annotation = self.annotations_dict[annotation_id]
+            # Remove from image annotations dict
+            if annotation.image_path in self.image_annotations_dict:
+                self.image_annotations_dict[annotation.image_path].remove(annotation)
+                if not self.image_annotations_dict[annotation.image_path]:
+                    del self.image_annotations_dict[annotation.image_path]
+
             # Delete the annotation
             annotation.delete()
             del self.annotations_dict[annotation_id]
@@ -634,8 +654,20 @@ class AnnotationWindow(QGraphicsView):
                 del self.annotations_dict[annotation.id]
 
     def delete_image_annotations(self, image_path):
-        annotations = self.get_image_annotations(image_path)
-        self.delete_annotations(annotations)
+        """Efficiently delete all annotations for a given image path"""
+        if image_path in self.image_annotations_dict:
+            annotations = self.image_annotations_dict[image_path]
+            # Delete graphics items and annotations in batch
+            for annotation in annotations:
+                annotation.delete()
+                del self.annotations_dict[annotation.id]
+                self.annotationDeleted.emit(annotation.id)
+
+            del self.image_annotations_dict[image_path]
+
+            # Clear confidence window if needed
+            if self.current_image_path == image_path:
+                self.main_window.confidence_window.clear_display()
 
     def delete_image(self, image_path):
         # Delete all annotations associated with image path
@@ -643,10 +675,11 @@ class AnnotationWindow(QGraphicsView):
         # Delete the image
         if self.current_image_path == image_path:
             self.scene.clear()
+            self.main_window.confidence_window.clear_display()
             self.current_image_path = None
             self.image_pixmap = None
             self.rasterio_image = None
-            self.active_image = False  # Reset image_set flag
+            self.active_image = False
 
     def clear_scene(self):
         # Clean up
