@@ -143,8 +143,10 @@ class Label(QWidget):
 
     @classmethod
     def from_dict(cls, data):
-        color = QColor(*data['color'])
-        return cls(data['short_label_code'], data['long_label_code'], color)
+        return cls(data['short_label_code'], 
+                   data['long_label_code'], 
+                   QColor(*data['color']),
+                   data['id'])
 
     def __repr__(self):
         return (f"Label(id={self.id}, "
@@ -202,6 +204,13 @@ class LabelWindow(QWidget):
         self.top_bar.addWidget(self.edit_label_button)
 
         self.top_bar.addStretch()  # Add stretch to the right side
+        
+        # Search bar for labels
+        self.search_bar = QLineEdit()
+        self.search_bar.setPlaceholderText("Search Labels")
+        self.search_bar.textChanged.connect(self.filter_labels)
+        self.search_bar.setFixedWidth(150)
+        self.top_bar.addWidget(self.search_bar)
 
         # Lock button
         self.label_lock_button = QPushButton()
@@ -287,7 +296,9 @@ class LabelWindow(QWidget):
         if dialog.exec_() == QDialog.Accepted:
             short_label_code, long_label_code, color = dialog.get_label_details()
             if self.label_exists(short_label_code, long_label_code):
-                QMessageBox.warning(self, "Label Exists", "A label with the same short and long name already exists.")
+                QMessageBox.warning(self, 
+                                    "Label Exists",
+                                    "A label with the same short and long name already exists.")
             else:
                 new_label = self.add_label(short_label_code, long_label_code, color)
                 self.set_active_label(new_label)
@@ -456,14 +467,18 @@ class LabelWindow(QWidget):
         self.update_labels_per_row()
         self.reorganize_labels()
 
-        # Refresh the scene with the new label
-        self.annotation_window.set_image(self.annotation_window.current_image_path)
+        # Refresh the scene with the new label (if applicable)
+        current_image_path = self.annotation_window.current_image_path
+        if current_image_path:
+            self.annotation_window.set_image(current_image_path)
 
     def delete_label(self, label):
         if (label.short_label_code == "Review" and
                 label.long_label_code == "Review" and
                 label.color == QColor(255, 255, 255)):
-            QMessageBox.warning(self, "Cannot Delete Label", "The 'Review' label cannot be deleted.")
+            QMessageBox.warning(self, 
+                                "Cannot Delete Label", 
+                                "The 'Review' label cannot be deleted.")
             return
 
         if self.show_confirmation_dialog:
@@ -506,13 +521,24 @@ class LabelWindow(QWidget):
         if not self.active_label or self.label_locked:
             return
 
-        try:
-            current_index = self.labels.index(self.active_label)
-        except ValueError:
-            # If the active label is not in the list, set it to None
-            self.active_label = None
+        # Get all labels from grid
+        grid_labels = []
+        for i in range(self.grid_layout.count()):
+            item = self.grid_layout.itemAt(i)
+            if item and item.widget():
+                grid_labels.append(item.widget())
+
+        if not grid_labels:
             return
 
+        try:
+            current_index = grid_labels.index(self.active_label)
+        except ValueError:
+            # If active label not in grid, select first label
+            if grid_labels:
+                self.set_active_label(grid_labels[0])
+            return
+        
         if key == Qt.Key_W:
             new_index = current_index - self.labels_per_row
         elif key == Qt.Key_S:
@@ -524,8 +550,8 @@ class LabelWindow(QWidget):
         else:
             return
 
-        if 0 <= new_index < len(self.labels):
-            self.set_active_label(self.labels[new_index])
+        if 0 <= new_index < len(grid_labels):
+            self.set_active_label(grid_labels[new_index])
 
     def toggle_label_lock(self, checked):
         """Toggle between lock and unlock states"""
@@ -559,6 +585,45 @@ class LabelWindow(QWidget):
     def unlock_label_lock(self):
         # Triggers the signal to toggle_label_lock method
         self.label_lock_button.setChecked(False)
+        
+    def filter_labels(self, search_text):
+        """Filter labels based on the search text and rebuild the grid with matching labels"""
+        # Unselect the selected annotation
+        self.annotation_window.unselect_annotations()
+        # Unselect the active label
+        self.deselect_active_label()
+        
+        # Get the search text in lowercase
+        search_text = search_text.lower()
+        
+        # Clear all widgets from the grid layout
+        while self.grid_layout.count():
+            item = self.grid_layout.takeAt(0)
+            if item.widget():
+                item.widget().setParent(None)
+        
+        # Filter labels that match the search criteria
+        filtered_labels = [
+            label for label in self.labels 
+            if search_text in label.short_label_code.lower() 
+            or search_text in label.long_label_code.lower()
+        ]
+        
+        # Add matching labels to the grid
+        for i, label in enumerate(filtered_labels):
+            row = i // self.labels_per_row
+            col = i % self.labels_per_row
+            self.grid_layout.addWidget(label, row, col)
+            label.show()
+        
+        # If we have an active label that's no longer visible, select first visible label
+        if self.active_label and self.active_label not in filtered_labels:
+            if filtered_labels:
+                self.set_active_label(filtered_labels[0])
+            else:
+                self.active_label = None
+                self.delete_label_button.setEnabled(False)
+                self.edit_label_button.setEnabled(False)
 
 
 class AddLabelDialog(QDialog):
@@ -682,7 +747,9 @@ class EditLabelDialog(QDialog):
     def validate_and_accept(self):
         # Cannot edit Review
         if self.label.short_label_code == 'Review' and self.label.long_label_code == 'Review':
-            QMessageBox.warning(self, "Cannot Edit Label", "The 'Review' label cannot be edited.")
+            QMessageBox.warning(self, 
+                                "Cannot Edit Label", 
+                                "The 'Review' label cannot be edited.")
             return
 
         # Can map other labels to Review
@@ -690,7 +757,9 @@ class EditLabelDialog(QDialog):
         long_label_code = self.long_label_input.text().strip()
 
         if not short_label_code or not long_label_code:
-            QMessageBox.warning(self, "Input Error", "Both short and long label codes are required.")
+            QMessageBox.warning(self, 
+                                "Input Error", 
+                                "Both short and long label codes are required.")
             return
 
         existing_label = self.label_window.get_label_by_codes(short_label_code, long_label_code)
@@ -711,13 +780,20 @@ class EditLabelDialog(QDialog):
             if result == QMessageBox.Yes:
                 self.label_window.edit_labels(self.label, existing_label, delete_old=True)
                 self.accept()
-
-            return
         else:
+            # Update the label itself
             self.label.short_label_code = short_label_code
             self.label.long_label_code = long_label_code
             self.label.color = self.color
             self.label.update_label_color(self.color)
-            self.accept()
+            self.label.update()
 
+            # Force a refresh of the label window grid
+            self.label_window.update_labels_per_row()
+            self.label_window.reorganize_labels()
+            self.label_window.scroll_area.viewport().update()
+
+            # Update annotation window
             self.label_window.edit_labels(self.label, self.label, delete_old=False)
+            
+            self.accept()
