@@ -7,18 +7,22 @@ import gc
 import numpy as np
 
 import torch
+from torch.cuda import empty_cache
 from ultralytics.models.fastsam import FastSAMPredictor
 
-from qtrangeslider import QRangeSlider
+from superqt import QRangeSlider
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QApplication, QComboBox, QDialog, QFormLayout, QHBoxLayout,
                              QLabel, QMessageBox, QPushButton, QSlider, QSpinBox,
                              QVBoxLayout, QGroupBox)
 
-from torch.cuda import empty_cache
 
 from coralnet_toolbox.ResultsProcessor import ResultsProcessor
+
 from coralnet_toolbox.QtProgressBar import ProgressBar
+
+from coralnet_toolbox.utilities import open_image
+
 from coralnet_toolbox.Icons import get_icon
 
 
@@ -46,7 +50,7 @@ class DeployGeneratorDialog(QDialog):
         self.label_window = main_window.label_window
         self.annotation_window = main_window.annotation_window
         self.sam_dialog = None
-        
+
         self.setWindowIcon(get_icon("sam.png"))
         self.setWindowTitle("FastSAM Generator")
         self.resize(400, 325)
@@ -66,7 +70,7 @@ class DeployGeneratorDialog(QDialog):
 
         # Create the layout
         self.layout = QVBoxLayout(self)
-        
+
         # Setup the info layout
         self.setup_info_layout()
         # Setup the model layout
@@ -77,7 +81,7 @@ class DeployGeneratorDialog(QDialog):
         self.setup_buttons_layout()
         # Setup the status layout
         self.setup_status_layout()
-        
+
     def showEvent(self, event):
         """
         Handle the show event to update label options and sync uncertainty threshold.
@@ -89,21 +93,21 @@ class DeployGeneratorDialog(QDialog):
         self.initialize_uncertainty_threshold()
         self.initialize_iou_threshold()
         self.initialize_area_threshold()
-        
+
     def setup_info_layout(self):
         """
         Set up the layout and widgets for the info layout.
         """
         group_box = QGroupBox("Information")
         layout = QVBoxLayout()
-        
+
         # Create a QLabel with explanatory text and hyperlink
         info_label = QLabel("Choose a Generator to deploy and use.")
-        
+
         info_label.setOpenExternalLinks(True)
         info_label.setWordWrap(True)
         layout.addWidget(info_label)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
@@ -113,10 +117,10 @@ class DeployGeneratorDialog(QDialog):
         """
         group_box = QGroupBox("Models")
         layout = QVBoxLayout()
-        
+
         self.model_combo = QComboBox()
         self.model_combo.setEditable(True)
-        
+
         # Define available models
         self.models = {
             "FastSAM-s": "FastSAM-s.pt",
@@ -129,7 +133,7 @@ class DeployGeneratorDialog(QDialog):
 
         layout.addWidget(QLabel("Select Model:"))
         layout.addWidget(self.model_combo)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
 
@@ -139,14 +143,14 @@ class DeployGeneratorDialog(QDialog):
         """
         group_box = QGroupBox("Parameters")
         layout = QFormLayout()
-        
+
         # Resize image dropdown
         self.resize_image_dropdown = QComboBox()
         self.resize_image_dropdown.addItems(["True", "False"])
         self.resize_image_dropdown.setCurrentIndex(0)
         self.resize_image_dropdown.setEnabled(False)  # Grey out the dropdown
         layout.addRow("Resize Image:", self.resize_image_dropdown)
-        
+
         # Image size control
         self.imgsz_spinbox = QSpinBox()
         self.imgsz_spinbox.setRange(512, 65536)
@@ -166,7 +170,7 @@ class DeployGeneratorDialog(QDialog):
         self.uncertainty_threshold_label = QLabel(f"{self.uncertainty_thresh:.2f}")
         layout.addRow("Uncertainty Threshold", self.uncertainty_threshold_slider)
         layout.addRow("", self.uncertainty_threshold_label)
-        
+
         # IoU threshold controls
         self.iou_thresh = self.main_window.get_iou_thresh()
         self.iou_threshold_slider = QSlider(Qt.Horizontal)
@@ -178,7 +182,7 @@ class DeployGeneratorDialog(QDialog):
         self.iou_threshold_label = QLabel(f"{self.iou_thresh:.2f}")
         layout.addRow("IoU Threshold", self.iou_threshold_slider)
         layout.addRow("", self.iou_threshold_label)
-        
+
         # Area threshold controls
         min_val, max_val = self.main_window.get_area_thresh()
         self.area_thresh_min = int(min_val * 100)
@@ -192,20 +196,22 @@ class DeployGeneratorDialog(QDialog):
         self.area_threshold_label = QLabel(f"{self.area_thresh_min:.2f} - {self.area_thresh_max:.2f}")
         layout.addRow("Area Threshold", self.area_threshold_slider)
         layout.addRow("", self.area_threshold_label)
-        
+
         # Max detections spinbox
         self.max_detections_spinbox = QSpinBox()
         self.max_detections_spinbox.setRange(1, 10000)
         self.max_detections_spinbox.setValue(self.max_detect)
         label = QLabel("Max Detections")
         layout.addRow(label, self.max_detections_spinbox)
-                
+
         # Task dropdown
         self.use_task_dropdown = QComboBox()
         self.use_task_dropdown.addItems(["Detect", "Segment"])
+        self.use_task_dropdown.currentIndexChanged.connect(self.update_task)
+        self.use_task_dropdown.currentIndexChanged.connect(self.deactivate_model)
         label = QLabel("Choose a task to perform")
         layout.addRow(label, self.use_task_dropdown)
-        
+
         # SAM dropdown
         self.use_sam_dropdown = QComboBox()
         self.use_sam_dropdown.addItems(["False", "True"])
@@ -213,41 +219,45 @@ class DeployGeneratorDialog(QDialog):
         label = QLabel("Use Predictor for creating Polygons:")
         label.setStyleSheet("font-weight: bold;")
         layout.addRow(label, self.use_sam_dropdown)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
     def setup_buttons_layout(self):
         """
         Setup action buttons in a group box.
         """
         group_box = QGroupBox("Actions")
         layout = QHBoxLayout()
-        
+
         load_button = QPushButton("Load Model")
         load_button.clicked.connect(self.load_model)
         layout.addWidget(load_button)
-        
+
         deactivate_button = QPushButton("Deactivate Model")
         deactivate_button.clicked.connect(self.deactivate_model)
         layout.addWidget(deactivate_button)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
     def setup_status_layout(self):
         """
         Setup status display in a group box.
         """
         group_box = QGroupBox("Status")
         layout = QVBoxLayout()
-        
+
         self.status_bar = QLabel("No model loaded")
         layout.addWidget(self.status_bar)
-        
+
         group_box.setLayout(layout)
         self.layout.addWidget(group_box)
-        
+
+    def update_task(self):
+        """Update the task based on the dropdown selection."""
+        self.task = self.use_task_dropdown.currentText().lower()
+
     def is_sam_model_deployed(self):
         """
         Check if the SAM model is deployed and update the checkbox state accordingly.
@@ -256,7 +266,7 @@ class DeployGeneratorDialog(QDialog):
         """
         if not hasattr(self.main_window, 'sam_deploy_model_dialog'):
             return False
-        
+
         self.sam_dialog = self.main_window.sam_deploy_model_dialog
 
         if not self.sam_dialog.loaded_model:
@@ -264,8 +274,8 @@ class DeployGeneratorDialog(QDialog):
             QMessageBox.critical(self, "Error", "Please deploy the SAM model first.")
             return False
 
-        return True 
-        
+        return True
+
     def initialize_uncertainty_threshold(self):
         """Initialize the uncertainty threshold slider with the current value"""
         current_value = self.main_window.get_uncertainty_thresh()
@@ -277,7 +287,7 @@ class DeployGeneratorDialog(QDialog):
         current_value = self.main_window.get_iou_thresh()
         self.iou_threshold_slider.setValue(int(current_value * 100))
         self.iou_thresh = current_value
-        
+
     def initialize_area_threshold(self):
         """Initialize the area threshold range slider"""
         current_min, current_max = self.main_window.get_area_thresh()
@@ -295,7 +305,7 @@ class DeployGeneratorDialog(QDialog):
     def update_iou_label(self, value):
         """Update IoU threshold and label"""
         value = value / 100.0
-        self.iou_thresh = value 
+        self.iou_thresh = value
         self.main_window.update_iou_thresh(value)
         self.iou_threshold_label.setText(f"{value:.2f}")
 
@@ -305,7 +315,12 @@ class DeployGeneratorDialog(QDialog):
         self.area_thresh_min = min_val / 100.0
         self.area_thresh_max = max_val / 100.0
         self.main_window.update_area_thresh(self.area_thresh_min, self.area_thresh_max)
-        self.area_threshold_label.setText(f"{self.area_thresh_min:.2f} - {self.area_thresh_max:.2f}")  
+        self.area_threshold_label.setText(f"{self.area_thresh_min:.2f} - {self.area_thresh_max:.2f}")
+
+    def get_max_detections(self):
+        """Get the maximum number of detections to return."""
+        self.max_detect = self.max_detections_spinbox.value()
+        return self.max_detect
 
     def load_model(self):
         """
@@ -319,41 +334,43 @@ class DeployGeneratorDialog(QDialog):
         try:
             # Get selected model path
             self.model_path = self.models[self.model_combo.currentText()]
-            self.max_detect = self.max_detections_spinbox.value()
             self.task = self.use_task_dropdown.currentText().lower()
 
             # Set the parameters
-            overrides = dict(model=self.model_path, 
-                             task=self.task, 
-                             mode='predict', 
-                             save=False, 
-                             max_det=self.max_detect,
+            overrides = dict(model=self.model_path,
+                             task=self.task,
+                             mode='predict',
+                             save=False,
+                             max_det=self.get_max_detections(),
                              imgsz=self.get_imgsz(),
-                             conf=0.00, 
-                             iou=1.00, 
+                             conf=self.main_window.get_uncertainty_thresh(),
+                             iou=self.main_window.get_iou_thresh(),
                              device=self.main_window.device)
-            
+
             # Load the model
             self.loaded_model = FastSAMPredictor(overrides=overrides)
-            
+            self.loaded_model.names = self.class_mapping
+
             with torch.no_grad():
                 # Run a blank through the model to initialize it
                 self.loaded_model(np.zeros((self.imgsz, self.imgsz, 3), dtype=np.uint8))
-            
+
             self.status_bar.setText(f"Model loaded: {self.model_path}")
             QMessageBox.information(self, "Model Loaded", "Model loaded successfully")
 
         except Exception as e:
             QMessageBox.critical(self, "Error Loading Model", str(e))
 
-        # Stop the progress bar
-        progress_bar.stop_progress()
-        progress_bar.close()
-        # Restore cursor
-        QApplication.restoreOverrideCursor()
+        finally:
+            # Restore cursor
+            QApplication.restoreOverrideCursor()
+            # Stop the progress bar
+            progress_bar.stop_progress()
+            progress_bar.close()
+
         # Exit the dialog box
         self.accept()
-        
+
     def get_imgsz(self):
         """Get the image size for the model."""
         self.imgsz = self.imgsz_spinbox.value()
@@ -368,67 +385,113 @@ class DeployGeneratorDialog(QDialog):
         """
         if not self.loaded_model:
             return
-   
+
+        # Create a results processor
+        results_processor = ResultsProcessor(
+            self.main_window,
+            self.class_mapping,
+            uncertainty_thresh=self.main_window.get_uncertainty_thresh(),
+            iou_thresh=self.main_window.get_iou_thresh(),
+            min_area_thresh=self.main_window.get_area_thresh_min(),
+            max_area_thresh=self.main_window.get_area_thresh_max()
+        )
+
         if not image_paths:
+            # Predict only the current image
             image_paths = [self.annotation_window.current_image_path]
-            
-        # Make cursor busy
+
         QApplication.setOverrideCursor(Qt.WaitCursor)
-            
-        progress_bar = ProgressBar(self.annotation_window, title="Making FastSAM Predictions")
-        progress_bar.show()
-        progress_bar.start_progress(len(image_paths))
+        try:
+            for image_path in image_paths:
+                inputs = self._get_inputs(image_path)
+                if inputs is None:
+                    continue
 
-        for image_path in image_paths:
-            
-            with torch.no_grad():
-                # Predict the image
-                results = self.loaded_model(image_path)
+                results = self._apply_model(inputs)
+                results = self._update_results(results, image_path)
+                results = self._apply_sam(results, image_path)
+                results = self._apply_tile_postprocessing(results)
+                self._process_results(results_processor, results)
+        except Exception as e:
+            print("An error occurred during prediction:", e)
+        finally:
+            QApplication.restoreOverrideCursor()
 
-                gc.collect()
-                empty_cache()
-            
-            # Update the results names
-            results[0].names = self.class_mapping
-                        
-            # Check if SAM model is deployed
-            if self.use_sam_dropdown.currentText() == "True":
-                # Apply SAM to the detection results
-                results = self.sam_dialog.predict_from_results(results, self.class_mapping)
-                        
-            # Update the progress bar
-            progress_bar.update_progress()
-            
-            # Create a results processor
-            results_processor = ResultsProcessor(self.main_window, 
-                                                 self.class_mapping,
-                                                 uncertainty_thresh=self.main_window.get_uncertainty_thresh(),
-                                                 iou_thresh=self.main_window.get_iou_thresh(),
-                                                 min_area_thresh=self.main_window.get_area_thresh_min(),
-                                                 max_area_thresh=self.main_window.get_area_thresh_max())
-            
-            if self.task.lower() == 'segment' or self.use_sam_dropdown.currentText() == "True":
-                results_processor.process_segmentation_results(results)
-            else:
-                results_processor.process_detection_results(results)
-                        
-        # Stop the progress bar
-        progress_bar.stop_progress()
-        progress_bar.close()
-                
-        # Make cursor normal
-        QApplication.restoreOverrideCursor()
         gc.collect()
         empty_cache()
 
+    def _get_inputs(self, image_path):
+        """Get the inputs for the model prediction."""
+        # Check if tile inference tool is enabled
+        if self.main_window.tile_inference_tool_action.isChecked():
+            inputs = self.main_window.tile_processor.make_crops(self.loaded_model, image_path)
+            if not inputs:
+                return None
+        else:
+            inputs = image_path
+        return inputs
+
+    def _apply_model(self, inputs):
+        """Apply the model to the inputs."""
+        # Update the model with user parameters
+        self.loaded_model.conf = self.main_window.get_uncertainty_thresh()
+        self.loaded_model.iou = self.main_window.get_iou_thresh()
+        self.loaded_model.max_det = self.get_max_detections()
+
+        # Make predictions
+        with torch.no_grad():
+            results = self.loaded_model(inputs)
+            gc.collect()
+            empty_cache()
+
+        # Return the results
+        yield results
+
+    def _update_results(self, results_generator, image_path):
+        """Update the results with the image path and class mapping."""
+        # Update the results with the image path and class mapping.
+        for results in results_generator:
+            for result in results:
+                if result:
+                    result.path = image_path
+                    result.names = self.class_mapping
+
+        return results
+
+    def _apply_sam(self, results, image_path):
+        """Apply SAM to the results if enabled."""
+        # Check if SAM is enabled
+        if self.use_sam_dropdown.currentText() == "True":
+            self.task = 'segment'
+            results = self.sam_dialog.predict_from_results(results, self.class_mapping, image_path)
+        return results
+
+    def _apply_tile_postprocessing(self, results):
+        """Apply tile postprocessing if needed."""
+        # Check if tile inference tool is enabled
+        if self.main_window.tile_inference_tool_action.isChecked():
+            results = self.main_window.tile_processor.detect_them(results, self.task == 'segment')
+        return results
+
+    def _process_results(self, results_processor, results):
+        """Process the results using the result processor."""
+        # Process the segmentations
+        if self.task.lower() == 'segment' or self.use_sam_dropdown.currentText() == "True":
+            results_processor.process_segmentation_results(results)
+        else:
+            results_processor.process_detection_results(results)
+
     def deactivate_model(self):
         """
         Deactivate the currently loaded model and clean up resources.
         """
         self.loaded_model = None
         self.model_path = None
+        # Clean up resources
         gc.collect()
         torch.cuda.empty_cache()
+        # Untoggle all tools
         self.main_window.untoggle_all_tools()
+        # Update status bar
         self.status_bar.setText("No model loaded")
-        QMessageBox.information(self, "Model Deactivated", "Model deactivated")
\ No newline at end of file
+        QMessageBox.information(self, "Model Deactivated", "Model deactivated")
