@@ -11,9 +11,9 @@ from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import (QApplication, QMessageBox, QVBoxLayout, QLabel, QDialog,
                              QDialogButtonBox, QGroupBox, QFormLayout, QLineEdit,
                              QDoubleSpinBox, QComboBox, QPushButton, QFileDialog, QSpinBox,
-                             QHBoxLayout)
+                             QHBoxLayout, QWidget)
 
-from coralnet_toolbox.Tile.QtCommon import TileSizeInput, OverlapInput, MarginInput
+from coralnet_toolbox.QtCommon import TileSizeInput, OverlapInput, MarginInput
 
 from coralnet_toolbox.QtProgressBar import ProgressBar
 
@@ -38,46 +38,83 @@ class Base(QDialog):
 
         self.setWindowIcon(get_icon("coral.png"))
         self.setWindowTitle("Tile Dataset")
-        self.resize(600, 100)
+        self.resize(800, 500)
 
-        # Object Detection / Instance Segmentation
-        self.annotation_type = None
+        # Main vertical layout
+        main_layout = QVBoxLayout(self)
 
-        self.layout = QVBoxLayout(self)
+        # Horizontal layout for columns
+        columns_layout = QHBoxLayout()
 
-        # Setup the info layout
+        # Create containers for left and right columns
+        left_container = QWidget()
+        right_container = QWidget()
+        left_layout = QVBoxLayout(left_container)
+        right_layout = QVBoxLayout(right_container)
+
+        # Create group boxes
+        self.info_group = QGroupBox()
+        self.tile_config_group = QGroupBox()
+        self.dataset_group = QGroupBox()
+        self.dataset_config_group = QGroupBox()
+
+        # Setup layouts
         self.setup_info_layout()
-        # Setup the dataset layout
-        self.setup_dataset_layout()
-        # Setup the tile config layout
         self.setup_tile_config_layout()
-        # Setup the dataset config layout
+        self.setup_dataset_layout()
         self.setup_dataset_config_layout()
-        # Setup the buttons layout
-        self.setup_buttons_layout()
+
+        # Add groups to column layouts
+        left_layout.addWidget(self.info_group)
+        left_layout.addWidget(self.tile_config_group)
+        right_layout.addWidget(self.dataset_group)
+        right_layout.addWidget(self.dataset_config_group)
+
+        # Add containers to columns layout
+        columns_layout.addWidget(left_container)
+        columns_layout.addWidget(right_container)
+
+        # Add columns to main layout
+        main_layout.addLayout(columns_layout)
+
+        # Add buttons
+        button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
+        button_box.accepted.connect(self.apply)
+        button_box.rejected.connect(self.reject)
+        main_layout.addWidget(button_box)
 
     def setup_info_layout(self):
-        """
-        Set up the layout and widgets for the info layout.
-        """
-        group_box = QGroupBox("Information")
+        """Set up the info layout."""
+        self.info_group.setTitle("Information")
         layout = QVBoxLayout()
 
-        # Create a QLabel with explanatory text and hyperlink
         info_label = QLabel("Tile an existing YOLO dataset into smaller non / overlapping images.")
-
         info_label.setOpenExternalLinks(True)
         info_label.setWordWrap(True)
         layout.addWidget(info_label)
 
-        group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
+        self.info_group.setLayout(layout)
+
+    def setup_tile_config_layout(self):
+        """Set up tile config layout."""
+        self.tile_config_group.setTitle("Tile Configuration Parameters")
+        layout = QFormLayout()
+
+        # Existing tile config code...
+        self.tile_size_input = TileSizeInput()
+        layout.addRow(self.tile_size_input)
+
+        self.overlap_input = OverlapInput()
+        layout.addRow(self.overlap_input)
+
+        self.margins_input = MarginInput()
+        layout.addRow(self.margins_input)
+
+        self.tile_config_group.setLayout(layout)
 
     def setup_dataset_layout(self):
-        """
-        Set up the dataset layout.
-        """
-        group_box = QGroupBox("Dataset Parameters")
+        """Set up dataset layout."""
+        self.dataset_group.setTitle("Dataset Parameters")
         layout = QFormLayout()
 
         # Source Directory
@@ -102,36 +139,11 @@ class Base(QDialog):
         dst_layout.addWidget(self.dst_button)
         layout.addRow("Destination Directory:", dst_layout)
 
-        group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
+        self.dataset_group.setLayout(layout)
 
-    def setup_tile_config_layout(self):
-        """
-        Set up the tile config parameters layout.
-        """
-        group_box = QGroupBox("Tile Configuration Parameters")
-        layout = QFormLayout()
-
-        # Tile Size
-        self.tile_size_input = TileSizeInput()
-        layout.addRow(self.tile_size_input)
-
-        # Overlap
-        self.overlap_input = OverlapInput()
-        layout.addRow(self.overlap_input)
-
-        # Margins
-        self.margins_input = MarginInput()
-        layout.addRow(self.margins_input)
-
-        group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
-        
     def setup_dataset_config_layout(self):
-        """
-        Set up the dataset configuration parameters layout.
-        """
-        group_box = QGroupBox("Dataset Configuration Parameters")
+        """Set up dataset config layout."""
+        self.dataset_config_group.setTitle("Dataset Configuration Parameters")
         layout = QFormLayout()
 
         # Image Extensions
@@ -203,20 +215,15 @@ class Base(QDialog):
         advanced_layout.addWidget(QLabel("Smoothing:"))
         advanced_layout.addWidget(self.smoothing_tolerance_spinbox)
         layout.addRow("Advanced Parameters:", advanced_layout)
-        
-        group_box.setLayout(layout)
-        self.layout.addWidget(group_box)
 
-    def setup_buttons_layout(self):
-        """
-        Set up the layout with buttons.
-        """
-        # Create a button box for the buttons
+        self.dataset_config_group.setLayout(layout)
+
+    def setup_buttons_layout(self, layout):
+        """Set up buttons layout."""
         button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
         button_box.accepted.connect(self.apply)
         button_box.rejected.connect(self.reject)
-
-        self.layout.addWidget(button_box)
+        layout.addWidget(button_box)
 
     def browse_src_dir(self):
         """
@@ -248,31 +255,6 @@ class Base(QDialog):
             return False
         return True
 
-    def validate_slice_wh(self, slice_wh):
-        """
-        Validate the slice_wh parameter to ensure it is a tuple of two integers.
-
-        :param slice_wh: Slice width and height
-        :return: True if valid, False otherwise
-        """
-        if not isinstance(slice_wh, tuple) or len(slice_wh) != 2 or not all(isinstance(i, int) for i in slice_wh):
-            QMessageBox.warning(self, "Invalid Tile Size", "The tile size must be a tuple of two integers.")
-            return False
-        return True
-
-    def validate_overlap_wh(self, overlap_wh):
-        """
-        Validate the overlap_wh parameter to ensure it is a tuple of two floats.
-
-        :param overlap_wh: Overlap width and height
-        :return: True if valid, False otherwise
-        """
-        correct_type = all(isinstance(i, (int, float)) for i in overlap_wh)
-        if not isinstance(overlap_wh, tuple) or len(overlap_wh) != 2 or not correct_type:
-            QMessageBox.warning(self, "Invalid Overlap", "The overlap must be a tuple of two floats.")
-            return False
-        return True
-
     def validate_ext(self, ext):
         """
         Validate the ext parameter to ensure it is a string and starts with a dot.
@@ -338,33 +320,6 @@ class Base(QDialog):
             return False
         return True
 
-    def validate_margins(self, margins):
-        """
-        Validate the margins parameter to ensure it is a valid type and value.
-
-        :param margins: Margins
-        :return: True if valid, False otherwise
-        """
-        if isinstance(margins, (int, float)):
-            if isinstance(margins, float) and not (0.0 <= margins <= 1.0):
-                QMessageBox.warning(self,
-                                    "Invalid Margins",
-                                    "The margin percentage must be between 0 and 1.")
-                return False
-            return True
-        elif isinstance(margins, tuple) and len(margins) == 4:
-            if all(isinstance(i, (int, float)) for i in margins):
-                if all(isinstance(i, float) for i in margins) and not all(0.0 <= i <= 1.0 for i in margins):
-                    QMessageBox.warning(self,
-                                        "Invalid Margins",
-                                        "All margin percentages must be between 0 and 1.")
-                    return False
-                return True
-        QMessageBox.warning(self,
-                            "Invalid Margins",
-                            "The margins must be a single integer, float, or a tuple of four integers/floats.")
-        return False
-
     def copy_class_mapping(self):
         """Checks to see if a class_mapping.json file exists in the source directory
         and copies it to the destination directory."""
@@ -404,10 +359,10 @@ class Base(QDialog):
         dst = os.path.join(self.dst_edit.text(), self.dst_name_edit.text())
 
         # Extract values
-        margins = self.margins_input.get_value()
-        slice_wh = self.tile_size_input.get_value()
-        overlap_wh = self.overlap_input.get_value(slice_wh[0], slice_wh[1])
-        
+        margins = self.margins_input.get_margins(validate=False)
+        slice_wh = self.tile_size_input.get_sizes(validate=False)
+        overlap_wh = self.overlap_input.get_overlap(validate=False, return_pixels=True)
+
         input_ext = self.input_ext_combo.currentText()
         output_ext = self.output_ext_combo.currentText()
         densify_factor = self.densify_factor_spinbox.value()
@@ -422,14 +377,11 @@ class Base(QDialog):
         # Perform all validation checks
         validation_checks = [
             (self.validate_source_directory(src), "Source directory validation failed"),
-            (self.validate_slice_wh(slice_wh), "Slice width/height validation failed"),
-            (self.validate_overlap_wh(overlap_wh), "Overlap width/height validation failed"),
             (self.validate_ext(input_ext), "Input extension validation failed"),
             (self.validate_ext(output_ext), "Output extension validation failed"),
             (self.validate_densify_factor(densify_factor), "Densify factor validation failed"),
             (self.validate_smoothing_tolerance(smoothing_tolerance), "Smoothing tolerance validation failed"),
             (self.validate_ratios(train_ratio, valid_ratio, test_ratio), "Dataset split ratios validation failed"),
-            (self.validate_margins(margins), "Margins validation failed")
         ]
 
         # Check if any validation failed
@@ -446,17 +398,17 @@ class Base(QDialog):
 
         def progress_callback(progress: TileProgress):
             title = f"Processing {progress.current_set_name.capitalize()} Set"
-            
+
             if progress.total_tiles:
                 progress_percentage = int((progress.current_tile_idx / progress.total_tiles) * 100)
                 title += f": {int(progress.current_image_idx/progress.total_images*100)}%"
             else:
                 progress_percentage = int((progress.current_image_idx / progress.total_images) * 100)
-            
+
             self.progress_bar.setWindowTitle(title)
             self.progress_bar.set_value(progress_percentage)
             self.progress_bar.update_progress()
-            
+
             if self.progress_bar.wasCanceled():
                 raise Exception("Tiling process was canceled by the user.")
 
