@@ -59,7 +59,13 @@ class SelectTool(Tool):
                 self.rectangle_selection = True
                 self.selection_start_pos = position
                 self.selection_rectangle = QGraphicsRectItem()
-                self.selection_rectangle.setPen(QPen(Qt.black, 2, Qt.DashLine))
+                # Calculate line thickness based on current viewing extent
+                extent = self.annotation_window.viewportToScene()
+                view_width = round(extent.width())
+                view_height = round(extent.height())
+                line_thickness = max(5, min(20, max(view_width, view_height) // 1000))
+                self.selection_rectangle.setPen(QPen(Qt.black, line_thickness, Qt.DashLine))
+                # Add the selection rectangle to the scene
                 self.annotation_window.scene.addItem(self.selection_rectangle)
 
             selected_annotation = self.select_annotation(position, items, event.modifiers())
@@ -195,7 +201,6 @@ class SelectTool(Tool):
         locked_label = self.get_locked_label()
         ctrl_pressed = modifiers & Qt.ControlModifier
 
-        # TODO this keeping the rectangle selection from *adding* all
         if selected_annotation in self.annotation_window.selected_annotations:
             if ctrl_pressed:
                 # Toggle selection when Ctrl is pressed
@@ -243,10 +248,7 @@ class SelectTool(Tool):
         new_center = selected_annotation.center_xy + delta
 
         if self.annotation_window.cursorInWindow(current_pos, mapped=True):
-            rasterio_image = self.annotation_window.rasterio_image
             self.annotation_window.set_annotation_location(selected_annotation.id, new_center)
-            selected_annotation.create_cropped_image(rasterio_image)
-            self.annotation_window.main_window.confidence_window.display_cropped_image(selected_annotation)
             self.move_start_pos = current_pos
 
     def handle_resize(self, current_pos):
