@@ -5,6 +5,7 @@ from PyQt5.QtWidgets import QApplication
 from coralnet_toolbox.QtMainWindow import MainWindow
 from coralnet_toolbox.utilities import console_user
 
+from coralnet_toolbox import __version__
 
 # ----------------------------------------------------------------------------------------------------------------------
 # Main Function
@@ -14,8 +15,8 @@ from coralnet_toolbox.utilities import console_user
 def run():
     try:
         app = QApplication([])
-        app.setStyle('WindowsXP')
-        main_window = MainWindow()
+        app.setStyle('Fusion')
+        main_window = MainWindow(__version__)
         main_window.show()
         app.exec_()
 
@@ -24,4 +25,4 @@ def run():
 
 
 if __name__ == '__main__':
-    run()
\ No newline at end of file
+    run()
